(* arhmm_eval.ml
 *
 * Copyright (C) 2006 Bertrand Mesot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA 02110-1301, USA.
 *)

open Misc;;
open Arhmm;;
open Mydataset;;
(*-------------------------------------------------------------------------*)
let nargs = 2;;
let usage_msg = "Usage: "^Sys.argv.(0)^"\n"^
  "\t[-q]\n"^
  "\t[-g]\n"^
  "\t<models list file>\n"^
  "\t<utterances list file>\n";;
(*-------------------------------------------------------------------------*)
let load_models fname =
  let list = ref [] in

  let file = open_in fname in
  let _ =
    try
      while true do
	let name = input_line file in
	let file = open_in name in
	let hmm = new arhmm () in

	hmm#load file;
	close_in file;

	list := (name, hmm) :: !list 
      done
    with
      End_of_file -> close_in file
  in
      
  Array.of_list (List.rev !list);;
(*-------------------------------------------------------------------------*)
let _ =
  (* Handle cmd line arguments *)

  let quiet = ref false
  and adapt = ref true in

  let optlist = 
    [ ("-q", Arg.Unit (fun _ -> quiet := true), 
       "Be quiet (false)") ;

      ("-g", Arg.Unit (fun () -> adapt := false), 
      "No gain adaptation (false)")
    ] 
  in

  let args = ref [] in
  Arg.parse optlist (fun s -> args := s :: !args) usage_msg;

  let args = Array.of_list (List.rev !args) in
  if Array.length args <> nargs then
    begin
      Arg.usage optlist usage_msg;
      exit 1
    end;

  let quiet = !quiet
  and adapt = !adapt in

  (* Load all models *)

  let models = load_models args.(0) in

  (* Compute the llh of each utterance with respect to each model *)

  let dset = new mydataset args.(1) in
  let size = dset#size ()
  and n    = Array.length models in

  for i=0 to size-1 do
    let fname = dset#get_fname i
    and seq   = dset#get i in

    if not quiet then 
      Printf.printf "%s: \n%!" fname
    else
      Printf.printf "%s: %!" fname;
    
    let llh = Array.init n (fun j ->
      let (_, m)    = models.(j) in
      let (_, x, _) = m#fwd ~adapt:adapt seq  in
      
      if not quiet then Printf.printf "\t%e\n%!" x;
      
      x)
    in

    (* Find the most likely model *)

    let max = ref llh.(0)
    and idx = ref 0 in

    for j=1 to n-1 do
      if llh.(j) > !max then
	begin
	  max := llh.(j);
	  idx := j
	end
    done;

    let (name, _) = models.(!idx) in
    if not quiet then 
      Printf.printf "\n\tBest is %s\n\n%!" name
    else
      Printf.printf "%s\n%!" name
  done;;
