(* arhmm_train.ml
 *
 * Copyright (C) 2006 Bertrand Mesot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA 02110-1301, USA.
 *)

open Misc;;
open Arhmm;;
open Mydataset;;
open Progressbar;;
(*-------------------------------------------------------------------------*)
let nargs = 3;;
let usage_msg = "Usage: "^Sys.argv.(0)^"\n"^
  "\t[-n <max number of iterations>]\n"^
  "\t[-t <threshold>]\n"^
  "\t[-g]\n"^
  "\t<input model file>\n"^
  "\t<utterances list file>\n"^
  "\t<output model file>\n";;

let pbar = new progressbar ();;
(*-------------------------------------------------------------------------*)
let _ =
  (* Handle cmd line arguments *)
  
  let n      = ref 1000000
  and thr    = ref 1e-7
  and adapt  = ref true
  and prefix = ref None in

  let arglist =
    [("-n", Arg.Int   (fun x -> n := x), 
      "Max number of iterations (1e6)");

     ("-t", Arg.Float (fun x -> thr := x), 
      "Threshold (1e-7)");

     ("-g", Arg.Unit  (fun () -> adapt := false), 
      "Turn gain adaptation off (false)");

     ("-p", Arg.String (fun s -> prefix := Some s), 
      "...") ]
  in
  
  let args = ref [] in
  Arg.parse arglist (fun s -> args := s :: !args) usage_msg;
  
  let args = Array.of_list (List.rev !args) in
  if Array.length args <> nargs then
    begin
      Arg.usage arglist usage_msg;
      exit 1
    end;

  let n      = !n
  and thr    = !thr
  and adapt  = !adapt
  and prefix = !prefix in

  (* Dataset *)

  let dset = new mydataset args.(1) in
  let size = dset#size () in

  (* Load the ARHMM *)

  let hmm  = new arhmm () in
  let file = open_in args.(0) in
  hmm#load file;
  close_in file;

  hmm#reset ();

  (* ... *)

  let rec loop i old_llh =
    pbar#init ~msg:(string_of_int i) size 80;
    
    let llh = ref 0. in
    
    for j=0 to size-1 do
      let seq = dset#get j in
      let tmp = hmm#acc ~adapt:adapt seq in
      
      llh := !llh +. tmp;
      
      pbar#update ~msg:(string_of_int i) j
    done;
    
    pbar#finish ~msg:(string_of_int i) 0;
    
    hmm#update ();
    hmm#reset ();
    
    let file = open_out args.(2) in
    hmm#save file;
    close_out file;

    begin
      match prefix with
      | None -> ()
      | Some name ->
	  let fname = name^"-"^(string_of_int i)^".dat" in
	  let file  = open_out fname in
	  hmm#save file;
	  close_out file;
    end;

    (* ... *)
    
    let llh = !llh /. (float_of_int size) in

    let ratio = (llh -. old_llh) /. abs_float(llh) in
    
    if ratio >= thr && i < n then loop (i+1) llh
  in
  loop 1 neg_infinity;

  Printf.printf "\n";;
  
