(* mydataset.ml
 *
 * Copyright (C) 2006 Bertrand Mesot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA 02110-1301, USA.
 *)

open Misc;;
open Dataset;;

class mydataset =
  fun ?(scale=1.) fname ->
    object (self)
      inherit [float array] dataset ()

      val mutable flist = [||]


      (* Public methods *)

      method size () = Array.length flist
      method iter f  = 
	Array.iter 
	  (fun x ->
	    let seq = self#getseq x in
	    f seq)
	  flist

      method iteri f =
	Array.iteri
	  (fun i x ->
	    Printf.printf "%s\n%!" x;
	    let seq = self#getseq x in
	    f i seq)
	  flist
	  
      method get i       = self#getseq flist.(i)
      method get_fname i = flist.(i) 


      (* Private methods *)

      method private getseq fname =
	let file = open_in fname in

	let rec loop seq =
	  try
	    let x = float_of_string (input_line file) in
	    loop (x :: seq)
	  with
	    End_of_file -> 
	      close_in file; 
	      List.rev seq
	in
	Array.of_list (loop [])
	  

      (* Initialiser *)

      initializer
	let file = open_in fname in

	let rec loop list =
	  try
	    let str = input_line file in
	    loop (str :: list)
	  with
	    End_of_file -> 
	      close_in file;
	      List.rev list
	in
	flist <- Array.of_list (loop [])
    end
