(* progressbar.ml
 *
 * Copyright (C) 2006 Bertrand Mesot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA 02110-1301, USA.
 *)

class progressbar =
  fun () ->
    object (self)
      val mutable size = 100
      val mutable width = 80
      val mutable frame = 0
      val mutable enabled = true

      val nframes = 4
      val animchar = [| '\\' ; '|' ; '/' ; '-' |]

      method init ?(msg = "") s w =
	size  <- s;
	width <- w-10;
	frame <- 0;

	if width < 1 then width <- 1;

	if enabled then self#display 0 msg
	
      method update ?(msg = "") pos =
	if enabled then self#display pos msg

      method finish ?(msg = "") n_ret =
	if enabled then
	  begin
	    self#display size msg;
	    
	    for i=0 to n_ret-1 do
	      Printf.printf "\n"
	    done
	  end

      method disable () = enabled <- false
      method enable ()  = enabled <- true;


      (* Private Methods *)

      method private display pos msg =
	let v = (float_of_int pos) /. (float_of_int size) in
	let p = (width * pos) / size in

	let sstr = Printf.sprintf "\r %s %c [" msg animchar.(frame) in
	let mstr = String.create width in

	for i=0 to p-1 do
	  String.set mstr i '='
	done;

	for i=p to width-1 do
	  String.set mstr i ' '
	done;

	let estr = Printf.sprintf "] %6.2f%%" (100.0 *. v) in
	Printf.printf "%s%!" (sstr ^ mstr ^ estr);

	frame <- (frame + 1) mod nframes
	
    end
